;;; vm-grepmail.el --- VM interface for grepmail
;; 
;; Copyright (C) 2001 Robert Fenk
;;
;; Author:      Robert.Fenk@gmx.de
;; Status:      Tested with XEmacs 21.1.10 & GNU Emacs 20.7 & VM 6.90
;; Keywords:    VM helpers
;; X-URL:       http://www.forwiss.de/~fenk/public/prog/
;; X-RCS:       $Id: vm-grepmail.el,v 1.4 2001/04/23 12:27:23 fenk Exp $

;;
;; This code is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:
;;
;; Add the following line to your .vm 
;;      (require 'vm-grepmail)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;###autoload
(defvar vm-grepmail-history
  nil
  "*History of previously used grepmail parameters.")

(defvar vm-grepmail-switches-history
  nil
  "*History of previously used grepmail parameters.")

(defvar vm-grepmail-command "grepmail"
  "*Path to the program.")

(defvar vm-grepmail-expression user-full-name
  "*Default expression for grepmail program.")

(defvar vm-grepmail-switches '("-q" "-m" "-R" "-e")
  "*Switches for grepmail program.")

(defvar vm-grepmail-folder-buffer nil)

;;;###autoload
(defun vm-grepmail (switches expression folders)
  "A not so excellent interface to grepmail.
Grepmail is a fast perl-script for finding mails which got lost in the
folder jungle.

SWITCHES should be a list of switches for grepmail.
EXPRESSION is searched for by grepmail.
FOLDERS should be a list of files/directories to search in."
  (interactive (list
                (split-string
                 (read-string "grepmail switches: "
                              (mapconcat 'identity vm-grepmail-switches " ")
                              vm-grepmail-switches-history))
                (read-string "grepmail expression: "
                             vm-grepmail-expression vm-grepmail-history)
                (let ((default (or vm-folder-directory "~/Mail"))
                      fd folders)
                  (while (not (string= fd default))
                    (setq fd (read-file-name
                              (format "folder/directory %s: "
                                      (if folders
                                          (concat "("
                                                  (mapconcat 'identity folders ", ") ")")
                                        ""))
                              default
                              default
                              t))
                    (if (not (string= fd default))
                        (setq folders (add-to-list 'folders fd))))
                  (if (null folders)
                      (setq folders (add-to-list 'folders fd)))
                  folders)))
  
  (let ((folder-buffer (format "* VM folder: grepmail %s %s %s *"
                               switches expression folders))
        (process-buffer (get-buffer-create
                         (format "* grepmail %s %s %s *"
                                 switches expression folders)))
        process)

    (when (get-buffer folder-buffer)
      (set-buffer folder-buffer)
      (if vm-summary-buffer (kill-buffer vm-summary-buffer))
      (if vm-presentation-buffer (kill-buffer vm-presentation-buffer))
      (kill-buffer folder-buffer))
    
    (setq folder-buffer (get-buffer-create folder-buffer))
    (set-buffer folder-buffer)
    (setq default-directory (expand-file-name vm-folder-directory))
    (setq buffer-read-only nil)
    (if vm-frame-per-folder (select-frame (make-frame)))
    (switch-to-buffer folder-buffer)
    (set-buffer-modified-p nil)
    (vm-mode)
    (font-lock-mode -1)
    (vm-update-summary-and-mode-line)
    (vm-display (current-buffer) t
                '(vm-scroll-forward vm-scroll-backward)
                '(reading-message))
    (vm-summarize t t)
    (vm-display (current-buffer) nil nil '(reading-message))
    (vm-display (current-buffer) t nil '(vm-next-message reading-message))

    (save-excursion
      (set-buffer process-buffer)
      (setq default-directory (expand-file-name vm-folder-directory))
      (erase-buffer)
      (switch-to-buffer process-buffer)
      (make-local-variable 'vm-grepmail-folder-buffer)
      (setq vm-grepmail-folder-buffer folder-buffer)

      (message "%S"
               (apply 'list
                      'start-process-shell-command "grepmail"
                      process-buffer
                      vm-grepmail-command
                      (append switches (list expression) folders)))

      (setq process
            (apply 'start-process-shell-command "grepmail"
                   process-buffer
                   vm-grepmail-command
                   (append switches (list expression) folders)))
      
      (if (null process)
          (error "Cannot start grepmail"))
      ;; set the send-filter
      (if vm-fsfemacs-p
          (set-buffer-process-coding-system 'raw-text-unix 'raw-text-unix))
      (set-process-filter process 'vm-grepmail-process-filter)
      (set-process-sentinel process 'vm-grepmail-process-done)
      process)))

(defun vm-grepmail-process-filter (process output)
  (set-buffer (process-buffer process))
  (goto-char (point-max))
  (insert output)
  (let (end)
    (goto-char (1+ (point-min)))
    (when (setq end (and (re-search-forward "^From " (point-max) t)
                         (match-beginning 0)))
      (vm-grepmail-grab-message (current-buffer) (point-min) end)
      (delete-region (point-min) end))))

(defun vm-grepmail-process-done (process state)
  (setq state (process-status process))
  (if (not (or (eq state 'exit) (eq state 'finished)
               (not (= (process-exit-status process) 0))))
      (error "grepmail terminated abnormally with %S %d"
             state (process-exit-status process)))
  
  (set-buffer (process-buffer process))
  (goto-char (point-max))
  (beginning-of-line)
  (vm-grepmail-grab-message (current-buffer) (point-min) (point))
  (let ((folder-buffer vm-grepmail-folder-buffer))
    (kill-this-buffer)
    (set-buffer folder-buffer)
    (vm-next-message 1)
    )
  (message "grepmail is finished."))

(defun vm-grepmail-grab-message (message-buffer start end)
  (save-excursion
    (set-buffer vm-grepmail-folder-buffer)
    (let ((buffer-read-only nil))
      (vm-save-restriction
       (widen)
       (goto-char (point-max))
       (insert-buffer-substring message-buffer start end)
       (cond ((eq major-mode 'vm-mode)
              (vm-clear-modification-flag-undos)))
        
       (vm-check-for-killed-summary)
       (vm-assimilate-new-messages)
       (vm-update-summary-and-mode-line)
       (set-buffer-modified-p nil))))
  (sit-for 0))

(provide 'vm-grepmail)
